/*	Traps.h - Trap generator and handler.
*
*******************************************************************************
* Copyright (c) 1996-97 Digi International Inc., All Rights Reserved          *
*                                                                             *
* This  software  contains  proprietary  and  confidential   information   of *
* Digi International  Inc.   By  accepting  transfer  of this copy, Recipient *
* agrees to retain this software in  confidence,  to  prevent  disclosure  to *
* others,  and  to make no  use of this software other than that for which it *
* was delivered.  This is an unpublished copyright work  Digi   International *
* Inc.    Except   as   permitted   by  federal  law,  17 USC 117, copying is *
* strictly prohibited.                                                        *
*                                                                             *
* Digi  International  Inc.  CONFIDENTIAL  -   (Digi    International    Inc. *
* Confidential  when   combined   with   the   aggregated  modules  for  this *
* product) OBJECT CODE ONLY SOURCE MATERIALS                                  *
*                                                                             *
* (C) COPYRIGHT Digi International Inc. 1996-97                               *
*                                                                             *
* All Rights Reserved                                                         *
*                                                                             *
* US   Government   Users   Restricted   Rights   -   Use,   duplication   or *
* disclosure  restricted by GSA ADP Schedule Contract with Digi International *
* Inc.                                                                        *
*                                                                             *
*******************************************************************************
*
*/

#if !defined(__TRAPS_H__)
#define	__TRAPS_H__

typedef struct
{
	AsnObjectIdentifier oiaEnterprise;
	AsnInteger iaGenericType;
	AsnInteger iaSpecificType;
	AsnTimeticks taTimeStamp;
	RFC1157VarBindList vblBindings;
}
Trap_t;	

typedef struct
{
	uint16 eType;
	uint32 iBoard;
	uint32 iGeneric1;
	uint32 iGeneric2;
}
TrapObject_t;

typedef struct
{
	AsnObjectIdentifier const * poiaEnterprise;
	AsnInteger iaGenericType;
	AsnInteger iaSpecificType;
	size_t iOffset;
	size_t cb;
}
TrapData_t;

typedef BOOL (*TRAP_HANDLER)(size_t const iBoard, void const * const pvData, TrapData_t const * const ptd);

BOOL InitTrapHandler(HANDLE const hHaveTraps);
void StartTrapHandler(void);
void StopTrapHandler(void);

BOOL AddTrap(TrapObject_t const * const pto, TrapData_t const * const ptd, TRAP_HANDLER pfnTrapHandler);
void RemoveTrap(TrapObject_t const * const pto, TrapData_t const * const ptd);

void TriggerTrap(Trap_t const * const pTrap);
Trap_t const * GetNextTrap(void);

#endif